@@ -1,9 +1,9 @@
 # -*- coding: UTF-8 -*-
 import logging
 import socket
+import time
 from msmart.security import security, MSGTYPE_HANDSHAKE_REQUEST, MSGTYPE_ENCRYPTED_REQUEST
-
-VERSION = '0.1.29'
+VERSION = '0.1.30'
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -18,16 +18,25 @@ class lan:
         self._socket = None
         self._token = None
         self._key = None
+        self._timestamp = time.time()
+        self._tcp_key = None
+        self._local = None
+        self._remote = device_ip + ":" + str(device_port)
 
     def _connect(self):
         if self._socket is None:
+            self._disconnect()
             _LOGGER.debug("Attempting new connection to {}:{}".format(
                 self.device_ip, self.device_port))
             self._buffer = b''
             self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+            # set timeout
             self._socket.settimeout(8)
             try:
                 self._socket.connect((self.device_ip, self.device_port))
+                self._timestamp = time.time()
+                self._local = ":".join(
+                    '%s' % i for i in self._socket.getsockname())
             except Exception as error:
                 _LOGGER.error("Connect Error: {}:{} {}".format(
                     self.device_ip, self.device_port, error))
@@ -37,6 +46,7 @@ class lan:
         if self._socket:
             self._socket.close()
             self._socket = None
+            self._tcp_key = None
 
     def request(self, message):
         # Create a TCP/IP socket
@@ -47,17 +57,24 @@ class lan:
                 _LOGGER.error("Sokcet is None: {}:{}".format(
                     self.device_ip, self.device_port))
                 return bytearray(0)
+            _LOGGER.debug("Socket {} -> {} tcp_key: {}".format(
+                self._local, self._remote, self._tcp_key))
             # Send data
-            _LOGGER.debug("Sending to {}:{} {}".format(
-                self.device_ip, self.device_port, message.hex()))
-            self._socket.send(message)
+            _LOGGER.debug(
+                "Sending to {} -> {} retries: {} message: {}".format(self._local, self._remote, self._retries, message.hex()))
+            self._socket.sendall(message)
 
             # Received data
             response = self._socket.recv(1024)
         except socket.error as error:
-            _LOGGER.error("Couldn't connect with Device {}:{} {}".format(
-                self.device_ip, self.device_port, error))
             self._disconnect()
+            self._retries += 1
+            if self._retries < 2:
+                _LOGGER.warn("About to send data again with Device {}:{} error: {}".format(
+                    self.device_ip, self.device_port, error))
+            else:
+                _LOGGER.error("Couldn't connect with Device {}:{} retries: {} error: {}".format(
+                    self.device_ip, self.device_port, self._retries, error))
             return bytearray(0)
         except socket.timeout:
             _LOGGER.error("Connect the Device {}:{} TimeOut for 8s. don't care about a small amount of this. if many maybe not support".format(
@@ -66,6 +83,11 @@ class lan:
             return bytearray(0)
         _LOGGER.debug("Received from {}:{} {}".format(
             self.device_ip, self.device_port, response.hex()))
+        if response == bytearray(0):
+            _LOGGER.warn("About to send data again with Device {}:{} response is null".format(
+                    self.device_ip, self.device_port))
+            self._disconnect()
+            self._retries += 1
         return response
 
     def authenticate(self, token: bytearray, key: bytearray):
@@ -77,8 +99,11 @@ class lan:
         response = self.request(request)[8:72]
         try:
             tcp_key = self.security.tcp_key(response, self._key)
+            self._tcp_key = tcp_key.hex()
             _LOGGER.info('Got TCP key for {}:{} {}'.format(
                 self.device_ip, self.device_port, tcp_key.hex()))
+            # After authentication, don’t send data immediately, so sleep 1s.
+            time.sleep(1)
         except Exception as error:
             self._disconnect()
             raise error
@@ -89,11 +114,26 @@ class lan:
         self.authenticate(self._token, self._key)
 
     def appliance_transparent_send_8370(self, data, msgtype=MSGTYPE_ENCRYPTED_REQUEST):
-        if self._socket is None:
+        socket_time = time.time() - self._timestamp
+        _LOGGER.debug("Data: {} msgtype: {} len: {} socket time: {}".format(data.hex(), msgtype, len(data), socket_time))
+        if self._socket is None or socket_time > 120:
+            if socket_time > 120:
+                _LOGGER.debug("Socket {} -> {} is TimeOut, Create New Socket ".format(self._local, self._remote))
+            self._disconnect()
             self._authenticate()
+        # copy from data in order to resend data
+        original_data = bytearray.copy(data)
         data = self.security.encode_8370(data, msgtype)
+        # time sleep retries second befor send data, default is 0 
+        time.sleep(self._retries)
+        responses = self.request(data)
+        _LOGGER.debug("Got responses len: {}".format(len(responses)))
+        if responses == bytearray(0) and self._retries < 2:
+            packets = self.appliance_transparent_send_8370(original_data, msgtype)
+            self._retries = 0
+            return packets
         responses, self._buffer = self.security.decode_8370(
-            self._buffer + self.request(data))
+            self._buffer + responses)
         packets = []
         for response in responses:
             if len(response) > 40 + 16:
@@ -102,8 +142,13 @@ class lan:
         return packets
 
     def appliance_transparent_send(self, data):
+        # time sleep retries second befor send data, default is 0 
+        time.sleep(self._retries)
         responses = self.request(data)
         _LOGGER.debug("Got responses len: {}".format(len(responses)))
+        if responses == bytearray(0) and self._retries < 2:
+            return self.appliance_transparent_send(data)
+        self._retries = 0
         if responses == bytearray(0):
             return responses
         packets = []
